# -*- coding: utf-8 -*-
# @Time    : 2021/1/28 20:40
# @Author  : Haijun


# -*- coding: utf-8 -*-
# @Time    : 2021/1/22 17:01
# @Author  : Haijun
import re

import aiohttp
import asyncio
import iso8601
import random
from dynaconf import settings
from loguru import logger
from aiostream import stream
from googleSearch.models.site_rank import SiteRank
from googleSearch.models.email import filter_email


class CrawlDetail(object):
    def __init__(self):
        self.headers = {
            'user-agent': random.choice(settings.USER_AGENT),
            'referer': 'https://www.google.com/'
        }

    async def crawl_product(self, expected_url):
        """
        获取最终的产品链接和店铺链接
        :param url:
        :return:
        """
        url = expected_url.get('url')
        image = expected_url.get('image')
        try:
            logger.info(f'开始爬取详情数据url==============={url}')
            async with aiohttp.TCPConnector(limit=10, verify_ssl=False) as tc:
                async with aiohttp.ClientSession(connector=tc) as session:
                    async with session.get(url=url, headers=self.headers, timeout=15) as req:
                        if req.status in [200, 201, 202, 203, 204, 205, 206, 300, 301, 302, 303, 304, 305, 306, 307]:
                            logger.info(f'详情页状态====={req.status} ===== 请求url==={url}=====响应url======{req.url}')
                            source = await req.text()
                            return await self.parse(source, image, str(req.url))
        except Exception as e:
            return []

    async def parse(self, res_text, image, res_url=None):
        # try:
        domain = re.findall(r'(.*?://.*?/)', res_url)[0]
        # 匹配商品上架时间
        _published_time = list()
        if 'Shopify.shop' in res_text or 'Shopify.theme' in res_text or \
                'Shopify.cdnHost' in res_text or 'Powered by Shopify' in res_text:
            if 'published_at":"' in res_text:
                p_at = re.findall(r'published_at":"(.*?)"', res_text)
                _published_time.extend(p_at)
            # elif 'created_at":"' in res_text:
            #     p_at = re.findall(r'created_at":"(.*?)"', res_text)
            #     _published_time.extend(p_at)
            elif 'created_at&quot;:&quot;' in res_text:
                p_at = re.findall(r'created_at&quot;:&quot;(.*?)&quot', res_text)
                _published_time.extend(p_at)
            elif '"datePublished":"' in res_text:
                p_at = re.findall(r'datePublished":"(.*?)"', res_text)
                _published_time.extend(p_at)
            published_time = self.create_time(_published_time)
            return await  self.source_filter_key(res_text, domain, image, res_url, 'shopify', published_time)
        elif 'content="WooCommerce' in res_text or 'woocommerce_params' in res_text or \
                'woocommerce-product-gallery' in res_text:
            if 'datePublished":"' in res_text:
                p_at = re.findall(r'datePublished":"(.*?)"', res_text)
                _published_time.extend(p_at)
            if 'modified_time" content="' in res_text:
                p_at = re.findall(r'modified_time" content="(.*?)"', res_text)
                _published_time.extend(p_at)
            if '"dateModified":"' in res_text:
                p_at = re.findall(r'"dateModified":"(.*?)"', res_text)
                _published_time.extend(p_at)
            published_time = self.create_time(_published_time)
            return await self.source_filter_key(res_text, domain, image, res_url, 'woocommerce', published_time)
        # except Exception as e:
        #     logger.error(f'爬取详情数据失败==============={res_url}===={e}')
        #     pass

    def create_time(self, p_time):
        """
        iso8601
        时间转换
        2020-06-05T15:01:50-04:00
        """
        published_time = None
        if p_time:
            published_time = p_time[0]
            t = iso8601.parse_date(published_time)
            published_time = t.strftime("%Y-%m-%d %H:%M:%S")
        return published_time

    async def source_filter_key(self, res_text, domain, image, res_url, source_key, published_time):
        try:
            # 当前页邮箱
            o_m = re.findall(r'([a-z][a-z0-9]*[-_]?[a-z0-9]*@[a-z0-9]*[-_]?[a-z0-9]*[\.][a-z]{2,3}[\.]?[a-z]{0,2})',
                             res_text)
            # 联系页邮箱
            xs = stream.iterate([f'{domain}pages/contact-us', f'{domain}pages/contact', f'{domain}pages/about-us',
                                 f'{domain}contact-us', f'{domain}contact', f'{domain}about-us'])
            # async for
            async for t in xs:
                email = await filter_email(t)
                if email:
                    o_m.append(email)
            _email = []
            if o_m:
                for i in o_m:
                    # 去除尾部的'.'
                    j = '.'.join([k for k in i.split('.') if k])
                    _email.append(j)
                # 去重
                o_m = list(set(_email))
                # 去除图片 xxx@xxx.jpg, xxx@xxx.png
                o_m = await self.wipe_email(o_m, 'png')
                o_m = await self.wipe_email(o_m, 'jpg')
                o_m = await self.wipe_email(o_m, 'jpeg')
                o_m = await self.wipe_email(o_m, 'webp')
                o_m = await self.wipe_email(o_m, 'gif')
                o_m = await self.wipe_email(o_m, 'bmp')

            # 排名 涉及到请求频次问题，此处省略，单独爬取
            # rank_data = await SiteRank().parse(domain) or None
            url_dict = {
                'host': domain,
                'image': image,
                'product_url': res_url,
                'tool_name': source_key,
                'email': o_m,
                'world_rank': None,  # 排名
                'published_time': published_time
            }
            logger.info(f'过滤后爬取数据结束============={url_dict}')
            return url_dict
        except Exception as e:
            print(e)

    async def wipe_email(self, e_l, i_type):
        """
        去除图片后缀的邮箱
        xxx@xxx.jpg
        :param e_l: 邮箱列表
        :param i_type: 图片格式
        :return:
        """
        expected_mail = list(
            set([i.strip() for i in e_l if i_type not in i and len(i) < 50]))
        return expected_mail


if __name__ == '__main__':
    data = {
        'image': 'https://cdn.shopify.com/s/files/1/0446/5108/5973/products/product-image-1434797904_480x480@2x.jpg?v%3D1598031340',
        'url': 'https://carolestore.com/products/led-makeup-mirror-light'
    }
    loop = asyncio.get_event_loop()
    r = loop.run_until_complete(CrawlDetail().crawl_product(data))
    print(r)



# -*- coding: utf-8 -*-
# @Time    : 2021/1/28 17:34
# @Author  : Haijun


import os
import random
import ssl
import requests
from urllib.request import Request
from urllib import request
import urllib.request
import urllib.parse
from http.cookiejar import LWPCookieJar
from dynaconf import settings
from googleSearch.utils.logger import logger
from requests_toolbelt.multipart.encoder import MultipartEncoder

home_folder = os.getenv('HOME')
if not home_folder:
    home_folder = os.getenv('USERHOME')
    if not home_folder:
        home_folder = '.'  # Use the current folder on error.
cookie_jar = LWPCookieJar(os.path.join(home_folder, '.google-cookie'))
try:
    cookie_jar.load()
except Exception:
    pass


class NoRedirHandler(request.HTTPRedirectHandler):
    # 禁止302跳转
    def http_error_302(self, req, fp, code, msg, headers):
        return fp

    http_error_301 = http_error_302


class UploadDataForSbi(object):
    """
    以图搜图获取sbi
    """

    def __init__(self):
        self.user_agent = random.choice(settings.USER_AGENT)

    def upload_image_url(self, image_url):
        """
        通过图片在线链接获取sbi
        """
        try:
            url = f'https://www.google.com/searchbyimage?image_url={image_url}&btnG=Search+by+image&encoded_image=&image_content=&filename=&hl=en'
            logger.info(url)
            request = Request(url)
            request.add_header('User-Agent', self.user_agent)
            cookie_jar.add_cookie_header(request)
            opener = urllib.request.build_opener(NoRedirHandler)  # 禁止重定向
            # context = ssl._create_unverified_context()
            response = opener.open(request)
            cookie_jar.extract_cookies(response, request)
            url_with_sbi = response.headers.get('Location')
            logger.info(f'通过图片链接获取sbi===={url_with_sbi}')
            response.close()
            try:
                cookie_jar.save()
            except Exception:
                logger.error('cookiejar保持失败')
            return url_with_sbi
        except Exception as e:
            logger.info(f'通过图片链接获取sbi失败====={e}')
            return None

    def upload_local_image(self, image_url):
        """
        通过上传本地图片获取sbi
        :param file_path: 图片本地路径
        :return:
        """
        try:
            # 目标url
            # file_path = init_data.get('image')
            upload_url = settings.BASE_URL + '/searchbyimage/upload'
            # 打开图片
            fp = open(image_url, 'rb')
            m = MultipartEncoder(
                {'encoded_image': (fp.name, fp, 'image/jpeg'), 'image_url': None, 'image_content': None, 'filename': None,
                 'btnG': None})
            # 此处需要禁止跳转
            response = requests.post(upload_url, data=m,
                                     headers={'Content-Type': m.content_type, 'User-Agent': self.user_agent},
                                     allow_redirects=False)
            url_with_sbi = response.headers.get('Location')
            logger.info(f'通过上传本地图片获取sbi========{url_with_sbi}')
            return url_with_sbi
        except Exception as e:
            logger.info(f'通过上传本地图片获取sbi失败========{e}')
            return None



# -*- coding: utf-8 -*-
# @Time    : 2021/1/28 17:29
# @Author  : Haijun
from googleSearch.utils.settings import load_or_create_settings
load_or_create_settings('')

from googleSearch.models.upload_data import UploadDataForSbi
from googleSearch.parsers.googleList import ParseGoogleList

__version__ = '0.3'
__all__ = [
    'UploadDataForSbi',
    'ParseGoogleList'
]



# -*- coding: utf-8 -*-
# @Time    : 2021/1/28 15:44
# @Author  : Haijun

import re
import aiohttp
import asyncio

headers = {
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.183 Safari/537.36'
}

async def filter_email(url):
    async with aiohttp.TCPConnector(limit=5, verify_ssl=False) as tc:
        async with aiohttp.ClientSession(connector=tc) as session:
            try:
                async with session.get(url=url, headers=headers, timeout=15) as req:
                    if req.status in [200, 301, 302]:
                        res_text = await req.text()
                        o_m = re.findall(
                            r'([A-Za-z][A-Za-z0-9]*[-_]?[A-Za-z0-9]*@[A-Za-z0-9]*[-_]?[A-Za-z0-9]*[\.][A-Za-z]{2,5}[\.]?[A-Za-z]{0,3})',
                            res_text)
                        return o_m
                    return False
            except Exception as e:
                return False



import redis
from pathlib import Path
from typing import Optional
from dynaconf import loaders, settings
from googleSearch.utils.logger import logger

DEFAULT_PATH = "settings.toml"
BASE_SETTINGS = {
    "IS_DEVE": True,
    "BASE_URL": "https://www.google.com",
    "HOME_DIR": str(Path().absolute()),
    "REDIS": {
        "HOST": 'localhost',
        "PORT": 6379,
        "DB": 4,
        "PASSWD": ""
    },
    "MYSQL": {
        "HOST": "",
        "USER": "",
        "PASSWD": "",
        "DATABASE": "",
        "PORT": 3306
    },
    'PROXY_USER': '',  # 代理用户名
    'PROXY_PWD': '',  # 代理密码
    "USER_AGENT": [
        "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.183 Safari/537.36"
    ],
    'EMAIL_EXISTS': ['admin@', 'sales@', 'team@', 'support@', 'customercare@', 'info@', 'customer@', 'service@',
                     'contact@', 'hello@'],
    'EXCLUDE_KEY_WORD': ['walmart.', 'sendo.', 'aliexpress.', 'wish.', 'amazon.', 'lazada.', 'ebay.', 'google.',
                         'shopee',
                         'tokopedia.', 'zipy.', 'tiki.', 'imall.', 'amazon.co.', 'joom.', 'dhgate.', 'taobao.',
                         'tmall.',
                         'jd.', 'suning.', 'zipy.', '1688.', 'pinterest', 'facebook.', 'youtube.', 'adobe.com', 'etsy.',
                         'alibaba.', 'vimeo.', 'tistory.', 'heavy.', 'line.', 'buzzfeed.', 'kijiji.', 'gearbest.',
                         'bonanza.', 'staples.', 'jimmyjazz.', 'sunsky-online.', 'pcwonderland.', 'pnghut.',
                         'aliradar.', 'pricecheck.', 'allmacworld.', 'wanelo.', 'findniche.', 'ecomhunt.', 'amz520.',
                         'jiwaji.'],
    'TOTAL_PAGE': 10,  # 期望爬取的页数
    # redis相关key
    # 用户输入AE url爬取初始数据
    'URL_FOR_CRAWL_AE_REDIS_KEY': 'URL_FOR_CRAWL_AE',
    # 上传图片后获取带sbi的链接
    'URL_WITH_SBI': 'URL_WITH_SBI',
    # GOOGLE搜索过滤后最终结果存入redis
    'GOOGLE_SEARCH_RESULT_REDIS_KEY': 'GOOGLE_SEARCH_RESULT'
}


def load_or_create_settings(path: Optional[str]):
    path = path or DEFAULT_PATH
    if not Path(path).exists():
        default_settings_path = str(Path.cwd() / Path(DEFAULT_PATH))
        logger.info(
            f'没有发现配置文件 "{Path(path).absolute()}". '
            f"创建默认配置文件: {default_settings_path}",
        )
        Path(path).parent.mkdir(parents=True, exist_ok=True)
        loaders.write(default_settings_path, BASE_SETTINGS, env="default")
    settings.load_file(path=path)
    logger.info("配置文件加载成功")



# -*- coding: utf-8 -*-
# @Time    : 2021/1/19 16:26
# @Author  : Haijun

from dynaconf import settings
import redis
import pymysql


class DatabaseConnection(object):
    try:
        redis_conn = redis.StrictRedis(host=settings.REDIS.HOST, port=settings.REDIS.PORT, db=settings.REDIS.DB,
                                       password=settings.REDIS.PASSWD)
        sql_conn = pymysql.connect(host=settings.MYSQL.HOST, user=settings.MYSQL.USER, password=settings.MYSQL.PASSWD,
                                   database=settings.MYSQL.DATABASE,
                                   port=settings.MYSQL.PORT)
    except Exception as e:
        pass


# -*- coding: utf-8 -*-
# @Time    : 2020/12/30 17:25
# @Author  : Haijun
import json
import re
import aiohttp
import asyncio
from spiders_utils.proxy import proxy

import requests


class SiteRank(object):
    def __init__(self):
        self.token_url = 'http://www.alexa.cn/rank/'
        self.api = 'http://www.alexa.cn/api/alexa/free'
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.183 Safari/537.36'
        }
        self.proxy = proxy('HQ86512757H93UVD', '711B84DE972CA59A')

    async def get_token(self, site):
        """
        获取token
        :param site:
        :return:
        """
        token_url = self.token_url + site
        try:
            async with aiohttp.TCPConnector(limit=5, verify_ssl=False) as self.tc:
                async with aiohttp.ClientSession(connector=self.tc) as self.session:
                    async with self.session.get(url=token_url, headers=self.headers, timeout=15,proxy=self.proxy['https']) as req:
                        res_text = await req.text()
        except Exception as e:
            async with self.session.get(url=token_url, headers=self.headers, timeout=15) as req:
                res_text = await req.text()
        token = re.search(r"token : '(.*?)'", res_text).group(1)
        return token

    async def parse(self, site):
        """
        获取排名
        :param site:
        :return:
        """
        site = site.replace('https://', '').replace('http://', '').replace('www.', '').replace('/', '')
        api_url = self.api + f'?token={await self.get_token(site)}&url={site}'
        try:
            async with aiohttp.TCPConnector(limit=5, verify_ssl=False) as self.tc:
                async with aiohttp.ClientSession(connector=self.tc) as self.session:
                    async with self.session.get(url=api_url, headers=self.headers,proxy=self.proxy['https']) as req:
                        res_dict = json.loads(await req.text())
        except Exception as e:
            async with self.session.get(url=api_url, headers=self.headers) as req:
                res_dict = json.loads(await req.text())
        if res_dict.get('status', None) == 0:
            data = res_dict.get('data', '')
            world_rank = data.get('world_rank')
            return world_rank
        return None


if __name__ == '__main__':
    """
        {
        "domain": "baidu.com",
        "world_rank": "4",
        "world_uv_rank": "5",
        "world_delta": "0",
        "country_rank": "2",
        "country_code": "CN",
        "update_time": "1609344676",
        "expire_time": "1609430400",
        "icp_area": "京",
        "flag": "public/flag/CN.png"
        }
    """
    loop = asyncio.get_event_loop()
    loop.run_until_complete(SiteRank().parse('https://www.baidu.com'))



# -*- coding: utf-8 -*-
# @Time    : 2021/1/28 20:15
# @Author  : Haijun

import os
import random
import ssl
import re
import asyncio
from urllib.request import Request, urlopen
from http.cookiejar import LWPCookieJar
from dynaconf import settings
from googleSearch.utils.logger import logger
from googleSearch.parsers.list_detail import CrawlDetail
from parsel import Selector
from googleSearch.utils.config import DatabaseConnection

cookie_jar = LWPCookieJar(os.path.join('.', '.google-cookie'))
try:
    cookie_jar.load()
except Exception:
    pass


class ParseGoogleList(object):
    def __init__(self):
        self.retry_times = 1

    def crawl_google_data(self, init_data):
        url = init_data['search_url']
        page = init_data['page']
        user_agent = random.choice(settings.USER_AGENT)
        request = Request(url)
        request.add_header('User-Agent', user_agent)
        request.add_header('accept',
                           'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9')
        cookie_jar.add_cookie_header(request)
        context = ssl._create_unverified_context()
        response = urlopen(request, context=context)
        cookie_jar.extract_cookies(response, request)
        res_text = response.read().decode('utf-8')
        response.close()
        try:
            cookie_jar.save()
        except Exception:
            pass
        loop = asyncio.get_event_loop()
        task = loop.create_task(self.parse_actions(init_data, res_text, page))
        loop.run_until_complete(task)
        result = task.result()
        return result


    async def parse_actions(self, init_data, res_text, n=0, is_retry=False):
        """
        解析
        :param init_data: 初始数据
        :param res_text: 响应文本
        :param n: 第几页
        :return:
        """
        # 用变量保存初始数据，用于下一页入库redis
        _init_data = init_data
        # 解析google列表
        html = Selector(res_text)
        data_list = html.xpath('//div[@id="rso"]/div[@class="g"]')
        logger.info(f'data_list====={len(data_list)}')
        # 只有本地上传图片的情况下，才不会有title
        title = init_data['title']
        if not title:
            init_data['title'] = await self.title(html)
        if data_list:
            cur_result = await self.parse_google_list(data_list)
            logger.info(f'当前页采集结果========{cur_result}')
            init_data['detail_list'].extend(cur_result)
            return init_data
        else:
            # 如果出现请求错误的情况，需要初始数据重新入redis，重新爬取
            error_msg = await self.error_msg(init_data, data_list, n, res_text)
            if error_msg and self.retry_times < 3:
                self.retry_times += 1
                logger.info(f'重试======url:{init_data["search_url"]}')
                self.crawl_google_data(init_data["search_url"])

        if not is_retry:
            # 下一页,如果是重试，就不取下一页
            next_page = await self.next_page(html)
            logger.info(f'下页======{next_page}')
            # 下一页入库
            if next_page:
                page_url = next_page.get('page_url')
                page = next_page.get('page')
                _init_data['page'] = page
                _init_data['search_url'] = page_url
                DatabaseConnection.redis_conn.rpush(settings.URL_WITH_SBI, _init_data)
                pass

    async def next_page(self, html):
        # 当前总页数
        page_url = html.xpath('//a[@id="pnnext"]/@href').get()
        logger.info(f'next page ====={page_url}')
        if page_url:
            page_no = re.findall(r'start=(\d+)', page_url)[0]
            page_url = f'{settings.BASE_URL}{page_url}'
            page = int(page_no) / 10
            if page <= settings.TOTAL_PAGE:
                # 如果存在下一页入库redis
                return {
                    "page_url": page_url,
                    'page': int(page)
                }
        return False

    async def parse_google_list(self, data_list) -> list:
        """
        获取列表图片和链接
        :param search_data:
        :return:
        """
        search_results = []
        for data in data_list:
            url = data.xpath("./div/div[1]/a/@href").get()
            image = data.xpath('./div/div[2]/div[1]/div/a/@href').get()
            if image:
                _image = re.search(r'/imgres\?imgurl=(.*?)\&', image)
                image = _image if _image else None
            data_dict = {
                "url": url,
                "image": image
            }
            filter_results = await self.filter_url(data_dict)
            if filter_results:
                search_results.append(filter_results)
        return search_results

    async def title(self, html) -> str:
        """
        标题，如果初始数据没有标题，就取google搜索框标题
        :param html:
        :return:
        """
        return html.xpath('//input[@title="Search"]/@value').get()

    async def filter_url(self, url):
        """
        过滤url
        :param url_list:
        :return:
        """
        # 存在黑名单
        del_url = [url for key in settings.EXCLUDE_KEY_WORD if key in url['url']]
        if not del_url:
            # _type = self.init_data['type']
            # 如果采集类型为关键字，需要是落地页，如果是以图搜图，不限制
            # if _type == 1:
            #     if '/products/' in url or '/product/' in url or '/collections/' in url:
            #         return await CrawlDetail().crawl_product(url)
            # elif _type == 2:
            #     return await CrawlDetail().crawl_product(url)
            return await CrawlDetail().crawl_product(url)
        return None

    async def error_msg(self, init_data, data_list, n, res_text):
        """
        错误回显
        :param data_list:
        :param n: 第几页
        :param res_text:
        :return:
        """
        # 异常
        if not data_list and n == 0:
            logger.error(f'解析为空===={init_data}')
            init_data['error_msg'] = f'解析为空,正在重试,请稍后查看)'
            init_data['code'] = 0
            return True
        # 搜索错误
        if 'Try different keywords' in res_text and n == 0:
            logger.error(f'第一页出现了Try different keywords===={init_data}')
            init_data['error_msg'] = '搜索错误,正在重试,请稍后查看)'
            init_data['code'] = 0
            return True
        elif 'The image is too big' in res_text and n == 0:
            logger.error(
                f'第一页出现了The image is too big 或者网络连接异常==={init_data}')
            init_data['error_msg'] = '网络连接失败,正在重试,请稍后查看'
            init_data['code'] = 0
            return True
        # 超出页码
        if 'Try different keywords' in res_text and n != 0:
            logger.error(f'第{n}页出现了Try different keywords=={init_data}')
            init_data['error_msg'] = '结果太少，未匹配到商品'
            init_data['code'] = 0
            return False
